--[[
	AG Filter - Temporarily block spammers for the current session

	Version:	1.1.3
	Date:		16 May 2010 - 29 May 2010
	Author:	DJarv

	Portions of this code were adapted from the following addons:
	- SpamFilter
]]


--============================
--= Settings, Defaults, and Local Variables
--============================
local LatestVersion = "1.1.3";

local AG_TextColour = "|cFF663300";   -- a nice brown
local AG_MenuColour = "|cFFCC6600";   -- a bit lighter so we can read it
local AG_ScanHighlight = "|cFFFF0000";   -- highlight colour for scan mode

local BlockedAuthors = {};
local RepeatBlockCount = 0;

local noRepeatMessages = {};
local noRepeatCursor = 1;

local lastLineID = -1;	-- last message processed
local lastLineAllow = true;

local calendarProcessing = false;

local GoldSpamerList = {
				 ["G4G"]	= "G O L D 4 G U I L D . C O M"
				,["MMOYY"]	= "OYY.COM"
				,["LKWAR"]	= "LUCKWAR.COM"
				,["MMO4S"]	= "MMO4STORE.C"
				,["GCAT"]	= "GOLDCAT.C"
				,["W4W"]	= "WOW4WOW.C"
				,["WGL"]	= "WOWGAMELIFE.C"
				,["mm4ss"]	= "MM4SS.C"
				,["FZF"]	= "FZF.c"
				,["KK4g"]	= "KK4G.C"
				};

AGFilterDefaults = {}
AGFilterDefaults.version = 1.2;
AGFilterDefaults.debugMode  = 0;
AGFilterDefaults.reportGold = false;
AGFilterDefaults.reportCalendar = false;
AGFilterDefaults.reportAnal = false;
AGFilterDefaults.notifyFlag = true;
AGFilterDefaults.noRepeatMessageMax = 20;
AGFilterDefaults.noRepeatTimeout = 120;
AGFilterDefaults.scan = "";

AGFilterSettings = {};



--============================
--= Initialisation Functions / Frames / Events
--============================

local AG_Frame = CreateFrame("frame");
AG_Frame:RegisterEvent("ADDON_LOADED");
--AG_Frame:RegisterEvent("CALENDAR_UPDATE_EVENT_LIST");
--AG_Frame:RegisterEvent("CALENDAR_UPDATE_PENDING_INVITES");

local function AGFilter_buildCircularList(theArray,theCursor,size)
	if (size < 1) then size=1 end;
	theCursor = 1;
	theArray = {};
	for x = 1, size do
		theArray[x] = {0,""};
	end
	return theArray,theCursor
end



--============================
--= CHAT_MSG_CHANNEL Filter Function
--============================
local function AGFilter_ChannelMsgFilter(frame, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
	local Msg = arg1:upper():gsub("|C[0-9A-F]+",""):gsub("|H[^|]+|H",""):gsub("|H|R","");
	local Author = arg2;		-- note: doesn't exist for system messages
	local ChannelID = arg8;
	local currentLineID = arg11;
	local currentTime = time();


	if lastLineID == currentLineID then		-- we have already checked this, do the same action
		if lastLineAllow then
			return false, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12
		else
			return true
		end
	else
		lastLineID = currentLineID;
		lastLineAllow = false;
	end

	if (ChannelID ~= 5 or AGFilterSettings.debugMode == 0) then	 -- for testing purposes - can't exactly spam trade.. only in debug mode will it work in 5
		if ((ChannelID ~= 2 and ChannelID ~= 1 and ChannelID ~= 3 and ChannelID ~= 4) or Author == UnitName("player")) then -- if the messsage is not on trade(2), general(1), local defense(3) LFG(4) or originating from us...
			lastLineAllow = true;
			return false, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12
		end
	end

	if (BlockedAuthors[Author] ~= nil) then	-- If this person has gold or anal spammed in this session
		BlockedAuthors[Author] = BlockedAuthors[Author] + 1;
		return true;
	end


	if (Msg:find("ANAL") ~= nil) then

		local foundWord = false; -- lets see if it's an actual word, or just part of a word, like 'analysis', scan whole message
		for word in string.gmatch(Msg,"(%u+)") do
			if word == "ANAL" then foundWord = true end;
		end

		if foundWord then
			BlockedAuthors[Author] = 1;
			if (AGFilterSettings.debugMode>0) then
				DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter not OK [" .. currentLineID .. "]: " .. Msg:lower());
			end
			if (AGFilterSettings.notifyFlag or AGFilterSettings.debugMode>0) then
				DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter blocked '" .. Author .. "' for anal spam.");
			end
			if (ChannelID ~= 5 and Author ~= UnitName("player")) then		-- we really dont want to complain about ourselves or people on the private channel, so nullify the debug channel
				if (AGFilterSettings.reportAnal) then
					ComplainChat(currentLineID);
				end
			end
			return true;
		end
	end

	for GSKey, GoldSpamTell in next, GoldSpamerList do
		if (Msg:find(GoldSpamTell) ~= nil) then
			BlockedAuthors[Author] = 1;
			if (AGFilterSettings.debugMode>0) then
				DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter not OK [" .. currentLineID .. "]: " .. Msg:lower());	
			end
			if (AGFilterSettings.notifyFlag or AGFilterSettings.debugMode>0) then
				DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter blocked '" .. Author .. "' for gold spam. [" .. GSKey .. "]");
			end
			if (ChannelID ~= 5 and Author ~= UnitName("player")) then		-- we really dont want to complain about ourselves here
				if (AGFilterSettings.reportGold) then
					ComplainChat(currentLineID);
				end
			end
			return true;
		end
	end

	if (AGFilterSettings.noRepeatMessageMax > 0) then		-- No-repeat system
		local currentTime = time();
		local expiredTime = currentTime - AGFilterDefaults.noRepeatTimeout;
		local compressedMsg = Author:upper() .. ":" .. Msg:gsub("[^%w^%]^%[]+","");
		local repeatedMsg = false;

		for msgKey,msgEntry in next,noRepeatMessages do
			if (msgEntry[1] > expiredTime and msgEntry[2] == compressedMsg) then
				repeatedMsg = true;
			end
		end
		if repeatedMsg then
			if (AGFilterSettings.debugMode>0) then
				DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter REPEAT [" .. currentLineID .. "]: " .. compressedMsg:sub(1,40):lower());
			end
			RepeatBlockCount = RepeatBlockCount + 1;
			return true;
		else
			noRepeatMessages[noRepeatCursor] = {currentTime,compressedMsg};
			if (noRepeatCursor >= AGFilterSettings.noRepeatMessageMax) then
				noRepeatCursor = 1;
			else
				noRepeatCursor = noRepeatCursor+1;
			end
		end
	end

	-- at this point we know the message is ok.

	if (AGFilterSettings.debugMode==2) then
		DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter OK [" .. currentLineID .. "]: " .. Msg:lower());
	end

	lastLineAllow = true;

	if (AGFilterSettings.scan ~= "") then				-- Scan system is on
		if (Msg:upper():gsub("[^%w^%]^%[]+",""):find(AGFilterSettings.scan) ~= nil) then  -- search for scanned.
			arg1 = AG_ScanHighlight .. arg1:gsub("|h|r","|h|r"..AG_ScanHighlight);       -- highlight the message.
			if (AGFilterSettings.debugMode>0) then
				DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter SCAN found '"..AGFilterSettings.scan.."'");
			end
			return false, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12      -- as we have changed the message, we need to do a special output.
		end
	end
	return false, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12


end

ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", AGFilter_ChannelMsgFilter);	-- And finally add the filter function to "channels" (meaning channels /1 to /99, does not apply to guilds, raids, say, yell etc).



--============================
--= Process calendar invites for spam
--============================

--local function AGFilter_processCalendarInvites()

--	local monthOffset,day,index = CalendarGetEventIndex();
--	local inviteCount = CalendarGetNumPendingInvites();
--	local i = 0;

--	DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "monthOffset [" .. monthOffset .. "]: day [" .. day .. "]: index [" .. index .. "]: inviteCount [" .. inviteCount .. "].");

--	--for i = 1, inviteCount do
--		CalendarOpenEvent(monthOffset, day, index);
--		local title, description, creator, eventType = CalendarGetEventInfo( );
--		DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. title .. "[" .. description .. "]");
--	--end	


--	--	CalendarOpenEvent(1, 13, 1);
--	--	local title, description, creator, eventType = CalendarGetEventInfo( );
--	--	DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. title .. "[" .. description .. "]");

--	return
--end




--============================
--= Register the Slash Command
--============================
SlashCmdList["AGFILTER"] = function(_msg)
	if (_msg) then
		local _, _, cmd, arg1 = string.find(string.upper(_msg), "([%w]+)%s*(.*)$");
		if ("RESET" == cmd) then		-- reset the list
			BlockedAuthors = {};
			RepeatBlockCount = 0;
			noRepeatMessages, noRepeatCursor = AGFilter_buildCircularList(noRepeatMessages, noRepeatCursor, AGFilterSettings.noRepeatMessageMax);
			local confirmMsg = "AGFilter: blocked list cleared and no-repeat filter restarted."
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		elseif ("DEFAULTS" == cmd) then		-- restore default options
			BlockedAuthors = {};
			RepeatBlockCount = 0;
			AGFilterSettings = AGFilterDefaults;
			noRepeatMessages, noRepeatCursor = AGFilter_buildCircularList(noRepeatMessages, noRepeatCursor, AGFilterSettings.noRepeatMessageMax);
			local confirmMsg = "AGFilter: default options restored. Reset also executed."
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		elseif ("NOTIFY" == cmd) then
			local confirmMsg = "AGFilter block notifications: ";
			if (tonumber(arg1)==1 or arg1=="ON") then
				AGFilterSettings.notifyFlag = true;
				confirmMsg = confirmMsg .. "set ON";
			elseif (tonumber(arg1)==0 or arg1=="OFF") then
				AGFilterSettings.notifyFlag = false;
				confirmMsg = confirmMsg .. "set OFF";
			else
				if (AGFilterSettings.notifyFlag) then
					confirmMsg = confirmMsg .. "currently ON";
				else
					confirmMsg = confirmMsg .. "currently OFF";
				end
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		elseif ("LIST" == cmd) then 	-- provide the list of blocked spammers
			local BlockedList = "";
			for aut, cnt in next, BlockedAuthors do
				BlockedList = BlockedList .. aut .. "[" .. cnt .. "]; "
			end
			if BlockedList == "" then BlockedList = "none"; end;
			local confirmMsg = "AGFilter currently blocking [msg count]: " .. BlockedList;
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
			confirmMsg = "AGFilter has hidden "..RepeatBlockCount.." duplicate messages.";
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		elseif ("REPORTGOLD" == cmd) then
			local confirmMsg = "AGFilter complain on gold spam: ";	
			if (tonumber(arg1)==1 or arg1=="ON") then
				AGFilterSettings.reportGold = true;
				confirmMsg = confirmMsg .. "set ON";
			elseif (tonumber(arg1)==0 or arg1=="OFF") then
				AGFilterSettings.reportGold = false;
				confirmMsg = confirmMsg .. "set OFF";
			else
				if (AGFilterSettings.reportGold) then
					confirmMsg = confirmMsg .. "currently ON";
				else
					confirmMsg = confirmMsg .. "currently OFF";
				end
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		elseif ("REPORTANAL" == cmd) then
			local confirmMsg = "AGFilter complain on anal spam: ";
			if (tonumber(arg1)==1 or arg1=="ON") then
				AGFilterSettings.reportAnal = true;
				confirmMsg = confirmMsg .. "set ON";
			elseif (tonumber(arg1)==0 or arg1=="OFF") then
				AGFilterSettings.reportAnal = false;
				confirmMsg = confirmMsg .. "set OFF";
			else
				if (AGFilterSettings.reportAnal) then
					confirmMsg = confirmMsg .. "currently ON";
				else
					confirmMsg = confirmMsg .. "currently OFF";
				end
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		elseif ("NOREPEAT" == cmd) then
			local confirmMsg = "AGFilter no-repeat filter: ";
			local NR_number,NR_timeout = arg1:match("^%s*(%d+):(%d+)%s*$");
			if (NR_number ~= nil and tonumber(NR_number)>=0 and tonumber(NR_number)<=50 and tonumber(NR_timeout)>=0 and tonumber(NR_timeout)<=999) then
				AGFilterSettings.noRepeatMessageMax = tonumber(NR_number);
				AGFilterSettings.noRepeatTimeout = tonumber(NR_timeout);
				noRepeatMessages, noRepeatCursor = AGFilter_buildCircularList(noRepeatMessages, noRepeatCursor, AGFilterSettings.noRepeatMessageMax);
				confirmMsg = confirmMsg .. "set to " ..AGFilterSettings.noRepeatMessageMax.."msgs:"..AGFilterSettings.noRepeatTimeout.."sec";
				if (AGFilterSettings.noRepeatMessageMax == 0) then
					confirmMsg = confirmMsg .. " [OFF]";
				end
			else
				confirmMsg = confirmMsg .. "currently " ..AGFilterSettings.noRepeatMessageMax.."msgs:"..AGFilterSettings.noRepeatTimeout.."sec";
				if (AGFilterSettings.noRepeatMessageMax == 0) then
					confirmMsg = confirmMsg .. " [OFF]";
				end
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		elseif ("SCAN" == cmd) then 	-- set a scan mode
			local confirmMsg = "AGFilter scan: ";	
			if (arg1=="OFF") then
				AGFilterSettings.scan = "";
				confirmMsg = confirmMsg .. "now disabled";
			elseif (arg1~="") then
				AGFilterSettings.scan = arg1:upper():gsub("[^%w^%]^%[]+","");
				confirmMsg = confirmMsg .. "'"..AGFilterSettings.scan.."'";
			else
				if (AGFilterSettings.scan == "") then
					confirmMsg = confirmMsg .. "currently disabled.";
				else
					confirmMsg = confirmMsg .. "currently scanning for '" .. AGFilterSettings.scan .. "'";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		elseif ("DEBUG" == cmd) then 	-- change debug mode on/off
			local confirmMsg = "AGFilter debug mode: ";	
			if (tonumber(arg1)==0 or arg1=="OFF") then
				AGFilterSettings.debugMode = 0;
				confirmMsg = confirmMsg .. "set 0:OFF";
			elseif (tonumber(arg1)==1 or arg1=="ON") then
				AGFilterSettings.debugMode = 1;
				confirmMsg = confirmMsg .. "set 1:ON";
			elseif (tonumber(arg1)==2 or arg1=="VERBOSE") then
				AGFilterSettings.debugMode = 2;
				confirmMsg = confirmMsg .. "set 2:VERBOSE";
			else
				confirmMsg = confirmMsg .. "currently " .. AGFilterSettings.debugMode;
				if (arg1 ~= "") then
					confirmMsg = confirmMsg .. " (option unknown: '"..arg1.."')";
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. confirmMsg);
		else -- either a gap command or an incorrect option
			local badCmd = "";
			if (_msg ~= "" and _msg ~= "?" and _msg ~= "HELP") then badCmd = " Unknown command: '" .. _msg .. "'"; end;

			local status = {};
			if (AGFilterSettings.notifyFlag) then status.notify="ON";     else status.notify="OFF";     end
			if (AGFilterSettings.reportGold) then status.reportGold="ON"; else status.reportGold="OFF"; end
			if (AGFilterSettings.reportAnal) then status.reportAnal="ON"; else status.reportAnal="OFF"; end
			if (AGFilterSettings.noRepeatMessageMax > 0) then status.noRepeat ="ON"; else status.noRepeat="OFF"; end
			if (AGFilterSettings.scan == "") then status.scan ="OFF"; else status.scan="'"..AGFilterSettings.scan.."'"; end
			status.noRepeat = AGFilterSettings.noRepeatMessageMax..":"..AGFilterSettings.noRepeatTimeout.." "..status.noRepeat;

			local helpMsg = "/AGFilter or /ag " .. badCmd .. "\n" ..
				"list: gives list of blocked spammers\n" ..
				"notify on/off: block notifications on/off [" .. status.notify .. "]\n" ..
				"reportgold on/off: gold spammer complaints on/off [" .. status.reportGold .. "]\n" ..
				"reportanal on/off: anal spammer complaints on/off [" .. status.reportAnal .. "]\n" ..
				"norepeat msgs:time : hide repeat msgs (0:0 for off, 50:999 max) [" .. status.noRepeat .. "]\n" ..
				"scan msg/off : enter a message to highlight [" .. status.scan .. "]\n" ..
				"reset: clears list of blocked spammers and restart no-repeat filter.\n" ..
				"defaults: restores default options and performs a reset.";
			if (AGFilterSettings.debugMode==1 or AGFilterSettings.debugMode==2) then
				helpMsg = helpMsg .. "\ndebug 0/1/2: change debug modes (0:off/1:on/2:verbose) [" .. AGFilterSettings.debugMode .. "]";
			end
			DEFAULT_CHAT_FRAME:AddMessage(AG_MenuColour .. helpMsg);
		end
	end
end

SLASH_AGFILTER1 = "/agfilter";
SLASH_AGFILTER2 = "/ag";


--============================
--= Loaded
--============================

local function AGFilter_OnEvent(self, event, arg1, ...)
	if ( event == "ADDON_LOADED" and arg1:lower() == "agfilter" ) then

		for AG_default_key,AG_default_value in next,AGFilterDefaults do
			if (AGFilterSettings[AG_default_key] == nil) then
				DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter UPDATE: [" .. AG_default_key .. "] added to saved variables");
				AGFilterSettings[AG_default_key] = AG_default_value;
			end
		end

		noRepeatMessages, noRepeatCursor = AGFilter_buildCircularList(noRepeatMessages, noRepeatCursor, AGFilterSettings.noRepeatMessageMax);

	--	AGFilter_processCalendarInvites();

		DEFAULT_CHAT_FRAME:AddMessage(AG_TextColour .. "AGFilter " .. LatestVersion .. " loaded.");

	--elseif ( event == "CALENDAR_UPDATE_EVENT_LIST" or event == "CALENDAR_UPDATE_PENDING_INVITES" ) then
	--
	--	if (not calendarProcessing) then
	--		calendarProcessing = true;
	--		AGFilter_processCalendarInvites();
	--		calendarProcessing = false;
	--	end

	end
end
AG_Frame:SetScript("OnEvent", AGFilter_OnEvent);

